# Drawing State-Of-Health channels and mass position

from typing import Tuple, Union, Dict

from sohstationviewer.view.util.plot_func_names import plot_functions

from sohstationviewer.model.general_data.general_data import GeneralData

from sohstationviewer.view.util.enums import LogType
from sohstationviewer.view.plotting.plotting_widget.\
    multi_threaded_plotting_widget import MultiThreadedPlottingWidget


class SOHWidget(MultiThreadedPlottingWidget):
    """
       Widget to display soh and mass position data.
       """

    def __init__(self, *args, **kwargs):
        MultiThreadedPlottingWidget.__init__(self, *args, **kwargs)

    def init_plot(self, d_obj: GeneralData, key: Union[str, Tuple[str, str]],
                  start_tm: float, end_tm: float, time_ticks_total: int):
        """
        Initiate plotting with gaps, top time bar

        :param d_obj: object of data
        :param key: data set's key
        :param start_tm: requested start time to read
        :param end_tm: requested end time to read
        :param time_ticks_total: max number of tick to show on time bar
        """
        self.data_object = d_obj
        self.plotting_data1 = d_obj.soh_data[key] if key else {}
        self.plotting_data2 = d_obj.mass_pos_data[key] if key else {}
        self.rt130_log_data = None
        if self.data_object.data_type == 'RT130':
            try:
                self.rt130_log_data = d_obj.log_data[key]['SOH'][0].split('\n')
            except KeyError:
                pass
        channel_list = d_obj.soh_data[key].keys() if key else []
        data_time = d_obj.data_time[key] if key else [0, 1]
        ret = super().init_plot(d_obj, data_time, key, start_tm, end_tm,
                                time_ticks_total, is_waveform=False)
        if not ret:
            return False

        not_found_chan = [c for c in channel_list
                          if c not in self.plotting_data1.keys()]
        if len(not_found_chan) > 0:
            msg = (f"The following channels is in Channel Preferences but "
                   f"not in the given data: {not_found_chan}")
            self.processing_log.append((msg, LogType.WARNING))
        return True

    def plot_single_channel(self, c_data: Dict, chan_id: str):
        """
        Plot the channel chan_id.

        :param c_data: data of the channel which includes down-sampled
            data in keys 'times' and 'data'. Refer to
            DataTypeModel.__init__.soh_data[key][chan_id] or
            DataTypeModel.__init__.mass_pos_data[key][chan_id]
        :param chan_id: name of channel
        """
        if len(c_data['times']) == 0:
            return
        chan_db_info = c_data['chan_db_info']
        plot_type = chan_db_info['plotType']
        ax = getattr(self.plotting, plot_functions[plot_type][1])(
            c_data, chan_db_info, chan_id)
        c_data['ax'] = ax
        ax.chan = chan_id
        self.axes.append(ax)
